@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Annotation;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Contains a single annotation from a JSON Schema evaluation
 
-our $VERSION = '0.525';
+our $VERSION = '0.526';
 
 use 5.020;
 use Moo;
@@ -16,7 +16,7 @@ no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
 use Safe::Isa;
 use MooX::TypeTiny;
-use Types::Standard qw(Str InstanceOf);
+use Types::Standard qw(Str InstanceOf Bool);
 use namespace::clean;
 
 has [qw(
@@ -41,6 +41,12 @@ has annotation => (
   required => 1,
 );
 
+has unknown => (
+  is => 'ro',
+  isa => Bool,
+  default => 0,
+);
+
 sub TO_JSON ($self) {
   return +{
     # note that locations are JSON pointers, not uri fragments!
@@ -68,7 +74,7 @@ JSON::Schema::Modern::Annotation - Contains a single annotation from a JSON Sche
 
 =head1 VERSION
 
-version 0.525
+version 0.526
 
 =head1 SYNOPSIS
 
@@ -111,6 +117,10 @@ actually fragmentless URI in most cases, as the indicated error will occur at a
 below the position where the most recent identifier had been declared in the schema. Further, if the
 schema never declared an absolute base URI (containing a scheme), this URI won't be absolute either.
 
+=head2 unknown
+
+A boolean flag, indicating whether the keyword is a known vocabulary keyword or unknown.
+
 =head2 annotation
 
 The actual annotation value (which may or may not be a string).
